"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _stream = require("stream");
var _streams = require("@verdaccio/streams");
var _commonsApi = require("@verdaccio/commons-api");
var _utils = require("./utils");
function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
class StoragePluginManager {
  constructor(config, packageName, logger, redisClient, db) {
    _defineProperty(this, "logger", void 0);
    _defineProperty(this, "packageName", void 0);
    _defineProperty(this, "config", void 0);
    _defineProperty(this, "redisClient", void 0);
    _defineProperty(this, "db", void 0);
    this.logger = logger;
    this.packageName = packageName;
    this.config = config;
    this.redisClient = redisClient;
    this.db = db;
  }

  /**
   * Handle a metadata update and
   * @param pkgName
   * @param updateHandler
   * @param onWrite
   * @param transformPackage
   * @param callback
   */
  updatePackage(pkgName, updateHandler, onWrite, transformPackage, callback) {
    this.logger.debug({
      pkgName
    }, '[verdaccio/redis] createPackage @{pkgName}');
    const key = _utils.REDIS_KEY.package + pkgName;
    this.redisClient.hget(key, _utils.PACKAGE_JSON_FILE).then(data => {
      // Check existence
      if (data === null) {
        callback((0, _commonsApi.getNotFound)(`package ${pkgName}'s package.json not found`));
        return;
      }
      // Parse JSON
      let parsed = null;
      try {
        parsed = JSON.parse(data);
      } catch (err) {
        callback((0, _commonsApi.getBadData)(`package ${pkgName}'s package.json is not a valid JSON`));
        return;
      }
      const pkg = parsed;
      // Update package
      updateHandler(pkg, err => {
        if (err) {
          callback((0, _utils.wrapError)(err));
          return;
        }
        const next = transformPackage(pkg);
        onWrite(pkgName, next, callback);
      });
    }).catch(err => callback((0, _utils.wrapError)(err)));
  }

  /**
   * Delete a specific file (tarball or package.json)
   * @param fileName
   * @param callback
   */
  deletePackage(fileName, callback) {
    this.deletePackageAsync(fileName).then(() => callback(null)).catch(err => callback(err));
  }
  async deletePackageAsync(fileName) {
    try {
      const pkgName = this.packageName;
      this.logger.debug({
        pkgName,
        fileName
      }, '[verdaccio/redis] deletePackage @{pkgName}/@{fileName}');
      const key = _utils.REDIS_KEY.package + pkgName;
      await this.redisClient.hdel(key, fileName);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Delete a package (folder, path)
   * This happens after all versions ar tarballs have been removed.
   * @param callback
   */
  removePackage(callback) {
    this.removePackageAsync().then(() => callback(null)).catch(err => callback(err));
  }
  async removePackageAsync() {
    try {
      const pkgName = this.packageName;
      this.logger.debug({
        pkgName
      }, '[verdaccio/redis] removePackage @{pkgName}');
      const key = _utils.REDIS_KEY.package + pkgName;
      await this.redisClient.del(key);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Publish a new package (version).
   * @param pkgName
   * @param pkg
   * @param callback
   */
  createPackage(pkgName, pkg, callback) {
    this.createPackageAsync(pkgName, pkg).then(() => callback(null)).catch(err => callback(err));
  }
  async createPackageAsync(pkgName, pkg) {
    try {
      this.logger.debug({
        pkgName
      }, '[verdaccio/redis] createPackage @{pkgName}');
      const key = _utils.REDIS_KEY.package + pkgName;
      // Check exist package
      const data = await this.redisClient.hget(key, _utils.PACKAGE_JSON_FILE);
      if (data !== null) {
        throw (0, _commonsApi.getConflict)(`package ${pkgName} already exist`);
      }
      await this.savePackageAsync(pkgName, pkg);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Perform write an object to the storage.
   * Similar to updatePackage but without middleware handlers
   * @param pkgName package name
   * @param pkg package metadata
   * @param callback
   */
  savePackage(pkgName, pkg, callback) {
    this.savePackageAsync(pkgName, pkg).then(() => callback(null)).catch(err => callback(err));
  }
  async savePackageAsync(pkgName, pkg) {
    try {
      this.logger.debug({
        pkgName
      }, '[verdaccio/redis] savePackage @{pkgName}');
      const key = _utils.REDIS_KEY.package + pkgName;
      const data = JSON.stringify(pkg);
      await this.redisClient.hset(key, _utils.PACKAGE_JSON_FILE, data);
      await this.db.setStat(pkgName);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Read a package from storage
   * @param pkgName package name
   * @param callback
   */
  readPackage(pkgName, callback) {
    this.readPackageAsync(pkgName).then(pkg => callback(null, pkg)).catch(err => callback(err));
  }
  async readPackageAsync(pkgName) {
    try {
      this.logger.debug({
        pkgName
      }, '[verdaccio/redis] readPackage @{pkgName}');
      const key = _utils.REDIS_KEY.package + pkgName;
      const data = await this.redisClient.hget(key, _utils.PACKAGE_JSON_FILE);
      if (data === null) {
        throw (0, _commonsApi.getNotFound)(`package ${pkgName}'s package.json not found`);
      }
      try {
        const pkg = JSON.parse(data);
        return pkg;
      } catch (err) {
        throw (0, _commonsApi.getBadData)(`package ${pkgName}'s package.json is not a valid JSON`);
      }
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Create writtable stream (write a tarball)
   * @param fileName
   */
  writeTarball(fileName) {
    const pkgName = this.packageName;
    this.logger.debug({
      pkgName,
      fileName
    }, '[verdaccio/redis] writeTarball @{pkgName}/@{fileName}');
    const uploadTarball = new _streams.UploadTarball({});
    const key = _utils.REDIS_KEY.package + pkgName;
    this.redisClient.hget(key, fileName).then(value => {
      if (value === null || value === '') {
        (0, _utils.bufferStreamToBase64String)(uploadTarball).then(data => {
          this.redisClient.hset(key, fileName, data).then(() => {
            uploadTarball.emit('success');
          }).catch(err => {
            uploadTarball.emit('error', (0, _utils.wrapError)(err));
          });
        }).catch(err => {
          uploadTarball.emit('error', (0, _utils.wrapError)(err));
        });
      } else {
        uploadTarball.emit('error', (0, _commonsApi.getConflict)(`tarball ${pkgName}/${fileName} already exist`));
      }
    }).catch(err => {
      uploadTarball.emit('error', (0, _utils.wrapError)(err));
    });
    return uploadTarball;
  }

  /**
   * Create a readable stream (read a from a tarball)
   * @param fileName
   */
  readTarball(fileName) {
    const pkgName = this.packageName;
    this.logger.debug({
      pkgName,
      fileName
    }, '[verdaccio/redis] readTarball @{pkgName}/@{fileName}');
    const readTarball = new _streams.ReadTarball({});
    const key = _utils.REDIS_KEY.package + pkgName;
    this.redisClient.hget(key, fileName).then(data => {
      if (data === null) {
        readTarball.emit('error', (0, _commonsApi.getNotFound)(`tarball ${pkgName}/${fileName} not found`));
        return;
      }
      const buf = Buffer.from(data, 'base64');
      const readable = _stream.Readable.from([buf]);
      readTarball.emit('content-length', buf.length);
      readTarball.emit('open');
      readable.pipe(readTarball);
      readable.on('error', err => {
        readTarball.emit('error', (0, _utils.wrapError)(err));
      });
      readTarball.abort = function () {
        readable.destroy((0, _commonsApi.getBadRequest)(`tarball ${pkgName}/${fileName} read has been aborted`));
      };
    }).catch(err => {
      readTarball.emit('error', (0, _utils.wrapError)(err));
    });
    return readTarball;
  }
}
exports.default = StoragePluginManager;