"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.dump = dump;
exports.dumpWithContext = dumpWithContext;
exports.restore = restore;
exports.restoreWithContext = restoreWithContext;
var _fs = _interopRequireWildcard(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _loglevel = _interopRequireDefault(require("loglevel"));
var _mkdirp = _interopRequireDefault(require("mkdirp"));
var _utils = require("./utils");
var _db = _interopRequireDefault(require("./db"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
_loglevel.default.setDefaultLevel('info');
/**
 * Parse redis config from command options
 * @param cmd
 */
function parseRedisConfig(opts) {
  if (opts.config) {
    const config = (0, _utils.parseConfigFile)(opts.config);
    _loglevel.default.info(`Load configuration at ${opts.config}`);
    if (config.store) {
      if (config.store['redis-storage']) {
        // parse store.redis-storage
        return config.store['redis-storage'];
      } else if (
      // parse store.storage-proxy.backends.redis-storage
      config.store['storage-proxy'] && config.store['storage-proxy']['backends'] && config.store['storage-proxy']['backends']['redis-storage']) {
        return config.store['storage-proxy']['backends']['redis-storage'];
      }
    }
    // return empty config
    return {};
  } else {
    if (opts.socket) return opts.socket;
    if (opts.url) return opts.url;else return {
      host: opts.host,
      port: opts.port ? parseInt(opts.port) : undefined,
      db: opts.db,
      password: opts.password,
      keyPrefix: opts.prefix
    };
  }
}

/**
 * Return a muted logger
 */
function getMutedLogger() {
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  const mute = () => {};
  const wrapper = {
    warn: mute,
    error: mute,
    info: mute,
    debug: mute,
    child: mute,
    http: mute,
    trace: mute
  };
  return wrapper;
}

/**
 * Return parsed command context.
 * @param dir
 * @param cmd
 */
async function getCommandContext(dir, opts) {
  // parse Redis config
  const mutedLogger = getMutedLogger();
  const redisConfig = parseRedisConfig(opts);
  const redisClient = (0, _utils.redisCreateClient)(redisConfig, mutedLogger);
  const db = new _db.default(redisClient, mutedLogger);
  const commandContext = {
    db,
    dir,
    logger: _loglevel.default,
    redisClient,
    includeTarball: opts.tarball,
    scan: opts.scan,
    dbName: opts.dbname || _utils.VERDACCIO_DB_FILE
  };
  // parse dir
  const absDir = _path.default.resolve(dir || '.');
  const stats = await _fs.promises.lstat(absDir);
  if (!stats) {
    throw new Error(`${dir} does not exist`);
  }
  if (!stats.isDirectory()) {
    throw new Error(`${dir} is not a directory`);
  }
  return commandContext;
}

/**
 * Dump database to the given directory
 * @param db
 * @param dir
 */
async function dumpDB({
  db,
  dbName,
  dir,
  logger
}) {
  logger.info('Dump database...');
  const secret = await db.getSecret();
  const packages = await db.get();
  packages.sort();
  const data = {
    list: packages,
    secret
  };
  const json = JSON.stringify(data);
  const filePath = _path.default.join(dir, dbName);
  logger.info(`  write ${filePath}`);
  await _fs.promises.writeFile(filePath, json, {
    encoding: 'utf8'
  });
}

/**
 * Dump packument and tarball to the given directory
 * @param db
 * @param dir
 */
async function dumpPackages({
  db,
  dir,
  redisClient,
  includeTarball,
  logger
}) {
  logger.info('Dump packages...');
  const packages = await db.get();
  for (const pkgName of packages) {
    // Create folder
    const pkgDir = _path.default.join(dir, pkgName);
    _mkdirp.default.sync(pkgDir);
    // Dump files
    const pkgContents = await redisClient.hgetall(_utils.REDIS_KEY.package + pkgName);
    for (const fileName of Object.keys(pkgContents)) {
      if (fileName != _utils.REDIS_FIELD.stat) {
        const filePath = _path.default.join(pkgDir, fileName);
        let fileContent = pkgContents[fileName];
        try {
          if (fileName == _utils.PACKAGE_JSON_FILE) {
            // format package.json with 2 spaces indent
            logger.info(`  write ${filePath}`);
            const json = JSON.parse(fileContent);
            fileContent = JSON.stringify(json, null, 2);
            await _fs.promises.writeFile(filePath, fileContent);
          } else if (includeTarball) {
            // write tarball
            logger.info(`  write ${filePath}`);
            const buf = Buffer.from(fileContent, 'base64');
            await _fs.promises.writeFile(filePath, buf);
          }
        } catch (err) {
          logger.error(`  failed to write ${filePath}, err: ${err}`);
        }
      }
    }
  }
}
async function dumpWithContext(commandContext) {
  await dumpDB(commandContext);
  await dumpPackages(commandContext);
}

/**
 * Dump Redis storage to the given directory
 * @param dir
 * @param cmd
 */
async function dump(dir, opts) {
  const commandContext = await getCommandContext(dir, opts);
  await dumpWithContext(commandContext);
}

/**
 * Scan dir and return found package names
 * @param dir
 * @param org
 */
async function scanPackageJson(dir, org) {
  const pkgNames = [];
  const names = await _fs.promises.readdir(dir);
  for (const name of names) {
    const filePath = _path.default.join(dir, name);
    const stats = await _fs.promises.lstat(filePath);
    if (stats.isDirectory()) {
      const dirPath = filePath;
      if (name.startsWith('@')) {
        const items = await scanPackageJson(dirPath, name);
        for (const item of items) {
          pkgNames.push(item);
        }
      } else {
        const packageJsonPath = _path.default.join(dirPath, _utils.PACKAGE_JSON_FILE);
        const stats = await _fs.promises.lstat(packageJsonPath);
        if (stats.isFile()) {
          const pkgName = org ? `${org}/${name}` : name;
          pkgNames.push(pkgName);
        }
      }
    }
  }
  pkgNames.sort();
  return pkgNames;
}

/**
 * Restore database from the given directory
 * @param db
 * @param dir
 */
async function restoreDB({
  db,
  dbName,
  dir,
  scan,
  logger
}) {
  logger.info('Restore database...');
  // parse dir
  const dbFilePath = _path.default.join(dir, dbName);
  logger.info(`  path: ${dbFilePath}`);
  const data = await _fs.promises.readFile(dbFilePath, {
    encoding: 'utf8'
  });
  const json = JSON.parse(data);
  const secret = json.secret;
  const packages = json.list;
  await db.setSecret(secret);
  logger.info('  set secret');
  for (const pkgName of packages) {
    logger.info(`  add ${pkgName}`);
    await db.add(pkgName);
  }
  if (scan) {
    logger.warn('Scan package.json...');
    logger.warn('(will move uplink packages to the database)');
    const pkgNames = await scanPackageJson(dir);
    for (const pkgName of pkgNames) {
      logger.info(`  add ${pkgName}`);
      packages.push(pkgName);
      await db.add(pkgName);
    }
  }
  return packages;
}

/**
 * Restore packument and tarball from the given directory
 * @param db
 * @param dir
 */
async function restorePackages(packages, {
  db,
  dir,
  redisClient,
  includeTarball,
  logger
}) {
  logger.info('Restore packages...');
  for (const pkgName of packages) {
    const pkgDir = _path.default.join(dir, pkgName);
    const redisKey = _utils.REDIS_KEY.package + pkgName;
    const pkgDirExist = _fs.default.existsSync(pkgDir);
    if (pkgDirExist) {
      logger.info(`  ${pkgName}`);
    } else {
      logger.error(`  failed to restore ${pkgName}, err: package folder doesn't exist`);
      continue;
    }
    // Restore package.json
    try {
      logger.info(`    ${_utils.PACKAGE_JSON_FILE}`);
      const packageJsonPath = _path.default.join(pkgDir, _utils.PACKAGE_JSON_FILE);
      const packageJsonContent = await _fs.promises.readFile(packageJsonPath, {
        encoding: 'utf8'
      });
      const packageJson = JSON.parse(packageJsonContent);
      await redisClient.hset(redisKey, _utils.PACKAGE_JSON_FILE, JSON.stringify(packageJson));
      let date = new Date();
      if (packageJson.time && packageJson.time.modified) {
        date = new Date(packageJson.time.modified);
      }
      await db.setStat(pkgName, date);
    } catch (err) {
      logger.error(`  failed to restore ${pkgName}'s ${_utils.PACKAGE_JSON_FILE}, err: ${err}`);
    }
    // Restore tarball
    if (includeTarball) {
      let names = await _fs.promises.readdir(pkgDir);
      names = names.filter(x => /\.(tgz|tar\.gz|tar)$/i.test(x));
      for (const name of names) {
        const filePath = _path.default.join(pkgDir, name);
        const stats = await _fs.promises.lstat(filePath);
        if (stats.isFile()) {
          logger.info(`    ${name}`);
          const buf = await _fs.promises.readFile(filePath);
          const tarballContext = buf.toString('base64');
          await redisClient.hset(redisKey, name, tarballContext);
        }
      }
    }
  }
}
async function restoreWithContext(commandContext) {
  const packages = await restoreDB(commandContext);
  await restorePackages(packages, commandContext);
}

/**
 * Restore Redis storage from the given directory
 * @param dir
 * @param cmd
 */
async function restore(dir, opts) {
  const commandContext = await getCommandContext(dir, opts);
  await restoreWithContext(commandContext);
}