"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _utils = require("./utils");
function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
/**
 * Database of Verdaccio. It's actually pretty simple as it only contains a secret
 * and a list of packages stored in Verdaccio.
 */
class Database {
  constructor(redisClient, logger) {
    _defineProperty(this, "redisClient", void 0);
    _defineProperty(this, "logger", void 0);
    this.redisClient = redisClient;
    this.logger = logger;
  }

  /**
   * Get the secret from the database
   */
  async getSecret() {
    try {
      this.logger.debug('[verdaccio/redis] getSecret');
      const secret = await this.redisClient.get(_utils.REDIS_KEY.secret);
      return secret || '';
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Update the secret in the database
   *
   * @param secret
   */
  async setSecret(secret) {
    try {
      this.logger.debug('[verdaccio/redis] setSecret');
      await this.redisClient.set(_utils.REDIS_KEY.secret, secret);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Search for packages based on the validation function
   *
   * @param validate
   */
  async search(validate) {
    try {
      this.logger.debug('[verdaccio/redis] db.search');
      const packages = await this.get();
      const result = packages.filter(validate);
      return result;
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Get stat object for package name
   *
   * @param name
   */
  async getStat(name) {
    try {
      this.logger.debug({
        name
      }, '[verdaccio/redis] db.getStat for @{name}');
      const result = await this.redisClient.hget(_utils.REDIS_KEY.package + name, _utils.REDIS_FIELD.stat);
      if (result === null) {
        return {
          name,
          time: 0,
          path: name
        };
      }
      const stat = JSON.parse(result);
      stat.name = name;
      return stat;
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Set stat object for package name
   *
   * @param name
   */
  async setStat(name, date) {
    try {
      this.logger.debug({
        name
      }, '[verdaccio/redis] db.setStat for @{name}');
      if (date === undefined) {
        date = new Date();
      }
      const stat = {
        name,
        path: name,
        time: date.getTime()
      };
      const data = JSON.stringify(stat);
      await this.redisClient.hset(_utils.REDIS_KEY.package + name, _utils.REDIS_FIELD.stat, data);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Get all the packages in the database
   */
  async get() {
    try {
      this.logger.debug('[verdaccio/redis] get');
      const result = await this.redisClient.smembers(_utils.REDIS_KEY.packages);
      return result;
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Add the given package to the database
   *
   * @param name
   */
  async add(name) {
    try {
      this.logger.debug({
        name
      }, '[verdaccio/redis] add @{name}');
      await this.redisClient.sadd(_utils.REDIS_KEY.packages, name);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Remove the given package from the database
   *
   * @param name
   */
  async remove(name) {
    try {
      this.logger.debug({
        name
      }, '[verdaccio/redis] remove @{name}');
      await this.redisClient.srem(_utils.REDIS_KEY.packages, name);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Save token to the database
   *
   * @param name
   */
  async saveToken(token) {
    try {
      const tokenId = `${token.user}:${token.key}`;
      this.logger.debug({
        tokenId
      }, '[verdaccio/redis] saveToken @{tokenId}');
      const key = _utils.REDIS_KEY.token + token.user;
      const data = JSON.stringify(token);
      await this.redisClient.hset(key, token.key, data);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Delete token from the database
   *
   * @param name
   */
  async deleteToken(user, tokenKey) {
    try {
      const tokenId = `${user}:${tokenKey}`;
      this.logger.debug({
        tokenId
      }, '[verdaccio/redis] deleteToken @{tokenId}');
      const key = _utils.REDIS_KEY.token + user;
      await this.redisClient.hdel(key, tokenKey);
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }

  /**
   * Read tokens from the database
   *
   * @param name
   */
  async readTokens(filter) {
    try {
      const user = filter.user;
      this.logger.debug({
        user
      }, '[verdaccio/redis] readTokens for @{user}');
      const key = _utils.REDIS_KEY.token + user;
      const result = await this.redisClient.hgetall(key);
      if (result === null) {
        return [];
      }
      return Object.values(result).map(x => {
        return JSON.parse(x);
      });
    } catch (err) {
      throw (0, _utils.wrapError)(err);
    }
  }
}
exports.default = Database;