"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VERDACCIO_DB_FILE = exports.TEST_REDIS_PREFIX = exports.REDIS_PREFIX = exports.REDIS_KEY = exports.REDIS_FIELD = exports.PACKAGE_JSON_FILE = void 0;
exports.bufferStreamToBase64String = bufferStreamToBase64String;
exports.parseConfigFile = parseConfigFile;
exports.redisCreateClient = redisCreateClient;
exports.wrapError = wrapError;
var _fs = _interopRequireDefault(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _ioredis = _interopRequireDefault(require("ioredis"));
var _commonsApi = require("@verdaccio/commons-api");
var _jsYaml = _interopRequireDefault(require("js-yaml"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function redisCreateClient(config, logger) {
  const client = new _ioredis.default(config);
  client.on('connect', function () {
    logger.info('[verdaccio/redis] connected to redis server');
  });
  client.on("ready", function () {
    logger.info("[verdaccio/redis] ready to use");
    logger.info("[verdaccio/redis] set enableOfflineQueue to false to make following web requests fail instantly when redis connection is down");
    client.options.enableOfflineQueue = false;
  });
  client.on('reconnecting', function (delay) {
    logger.info({
      delay
    }, '[verdaccio/redis] reconnecting in @{delay}ms');
  });
  client.on('end', function () {
    logger.info('[verdaccio/redis] redis connection end');
  });
  client.on('close', function () {
    logger.info('[verdaccio/redis] redis connection close');
  });
  client.on('error', function (err) {
    logger.error({
      err
    }, '[verdaccio/redis] redis error @{err}');
  });
  return client;
}
const REDIS_PREFIX = 've:';
exports.REDIS_PREFIX = REDIS_PREFIX;
const TEST_REDIS_PREFIX = 'testve:';
exports.TEST_REDIS_PREFIX = TEST_REDIS_PREFIX;
function prefix(key) {
  const prefix = process.env.NODE_ENV === 'test' ? TEST_REDIS_PREFIX : REDIS_PREFIX;
  return prefix + key;
}
const REDIS_KEY = {
  secret: prefix('secret'),
  packages: prefix('pkgs'),
  package: prefix('pkg:'),
  token: prefix('token:')
};
exports.REDIS_KEY = REDIS_KEY;
const REDIS_FIELD = {
  stat: 'stat'
};
exports.REDIS_FIELD = REDIS_FIELD;
const VERDACCIO_DB_FILE = '.verdaccio-db.json';
exports.VERDACCIO_DB_FILE = VERDACCIO_DB_FILE;
const PACKAGE_JSON_FILE = 'package.json';

/**
 * Wrap an generic error to a verdaccio error
 *
 * @param error
 */
exports.PACKAGE_JSON_FILE = PACKAGE_JSON_FILE;
function wrapError(err) {
  if (err instanceof Error) {
    if ('code' in err) return err;
    return (0, _commonsApi.getInternalError)(err.message);
  } else {
    return (0, _commonsApi.getInternalError)(String(err));
  }
}

/**
 * Load verdaccio config file
 * @param configPath
 */
function parseConfigFile(configPath) {
  try {
    const absPath = _path.default.resolve(configPath);
    if (/\.ya?ml$/i.test(absPath)) {
      return _jsYaml.default.safeLoad(_fs.default.readFileSync(absPath, 'utf8'));
    }
    return require(absPath);
  } catch (err) {
    const error = wrapError(err);
    if (error.code !== 'MODULE_NOT_FOUND') {
      error.message = 'failed to load verdaccio configuration file.';
    }
    throw error;
  }
}

/**
 * Convert buffer stream to base64 string
 */
function bufferStreamToBase64String(stream) {
  const chunks = [];
  return new Promise(function (resolve, reject) {
    stream.on('data', function (chunk) {
      chunks.push(chunk);
    });
    stream.on('end', function () {
      const buf = Buffer.concat(chunks);
      const data = buf.toString('base64');
      resolve(data);
    });
    stream.on('error', function (err) {
      reject(err);
    });
  });
}